import java.io.*;
import java.util.StringTokenizer;

import java.util.HashSet;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;

public class Multiply {

  // como carregar o vetor "v" em memória principal para todos os nós?

  public static double[] v;

  public static class MatrixMapper extends Mapper<Object, // input key type
                                            Text,    // input value type
                                            Text,    // output key type
                                            Text>    // output value type
  {
    public void map(Object key,     // input key -- ignorado aqui
                    Text value,     // input value -- contém o conteúdo de cada linha do arquivo matriz
                    Context context // context é onde os dados estão disponíveis para próxima etapa do MapCombineReduce
                    ) throws IOException, InterruptedException 
    {
      String record   = value.toString();
      String[] parts  = record.split(" ");

      IntWritable i = new IntWritable(new Integer(parts[0]));
      int j         = new Integer(parts[1]);
      Double mij    = new Double(parts[2]);

      double mijvj = mij * v[j];

      context.write(new Text(i.toString()), new Text(""+mijvj));
    }
  }

  // REDUCE
  public static class MultiplyReducer extends Reducer<Text, Text, 
                                                  Text, Text> {

@Override
    public void reduce(Text key, 
                       Iterable<Text> values, 
                       Context context) throws IOException, InterruptedException {

      double sum = 0;

      for (Text t: values) {
        Double d = new Double(t.toString());
        sum = sum + d;
      }

      Text result = new Text();
      result.set(""+sum);

      context.write(key, result);
    }
  }

  public static void readV(String filename, int N) throws Exception {

    v = new double[N];
    BufferedReader br = new BufferedReader(new FileReader(filename));

    String l;
    int i = 0;
    while ( (l = br.readLine()) != null ){
      v[i] = new Double(l);
    } 
    br.close();
  }

  public static void main(String[] args) throws Exception {
    Configuration conf = new Configuration();
    Job job = Job.getInstance(conf, "multiply");
    job.setJarByClass(Multiply.class);

    // INPUT and MAP
    job.setMapperClass(MatrixMapper.class);
    // REDUCE
    job.setReducerClass(MultiplyReducer.class); 

    //OUTPUT
    job.setOutputKeyClass  (Text.class);
    job.setOutputValueClass(Text.class);

    FileInputFormat.addInputPath(job, new Path(args[0]));
    String Vfilename = args[1];
    int N = new Integer(args[2]);

    readV(Vfilename, N);

    FileOutputFormat.setOutputPath(job, new Path(args[3]));

    System.exit(job.waitForCompletion(true) ? 0 : 1);
  }
}

